## PyFunge version 0.2 -- http://pyfunge.kldp.net/
## Copyright (c) 2004, Kang Seonghoon (Tokigun).
## This program is distributable under the MIT license.
##
## $Id: execute.py,v 1.4 2004/06/10 16:04:22 tokigun Exp $

"""execute module in PyFunge 0.2 -- http://pyfunge.kldp.net/
This program is distributable under the MIT license.
$Id: execute.py,v 1.4 2004/06/10 16:04:22 tokigun Exp $

This module implements Funge-98 (including Befunge-93) instructions.
There are 16 classes implements instruction corresponding to each Funge version.


Imported modules
================

From PyFunge packages:
    PyFunge.exception
    PyFunge.ip
    PyFunge.space

From standard packages:
    itertools - widely used in this module :)
    random - used in Unefunge93.command63()
    os - used in Unefunge98.command121() and Filesystem98.command61()
    time - used in Unefunge98.command121()


Table of hierarchy
==================

* shouldn't be instantiated.

FungeIP from PyFunge.ip
 |
 +-- FungeExec_Unefunge93 *
      |
      +-- FungeExec_Befunge93
      |    |
      +----|-- FungeExec_Unefunge98
           |    |
           +----+-- FungeExec_Befunge98
                |    |
                |    +-- FungeExec_Trefunge98
                |    |    |
                |    |    |   FungeExec_Filesystem98 *
                |    |    |    |
                |    |    +----+-- FungeExec_FTrefunge98
                |    +----|----+-- FungeExec_FBefunge98
                +----|----|----+-- FungeExec_FUnefunge98
                |    |    |    |
                |    |    |    |   FungeExec_Concurrent98 *
                |    |    |    |    |
                |    |    +----|----+-- FungeExec_CTrefunge98
                |    +----|----|----+-- FungeExec_CBefunge98
                +----|----|----|----+-- FungeExec_CUnefunge98
                |    |    |    |    |
                |    |    +----+----+-- FungeExec_FCTrefunge98
                |    +---------+----+-- FungeExec_FCBefunge98
                +--------------+----+-- FungeExec_FCUnefunge98


Class naming method
===================

- Above all, "name of class" means real name of class without prefix "FungeExec_".
  For instance, a class called "Befunge93" has real name "FungeExec_Befunge93".

- Names of all classes in this module ends with suffix "93" or "98".
  "93" means this class implements (Be)Funge-93 instructions, and so does "98".

- Most names of classes starts with prefix "Unefunge", "Befunge", "Trefunge".
  They mean the number of Funge-space dimension supported by each Funge version.
  Note that "Unefunge93" is base class of the others, but shouldn't be instantiated.
  (Actually it doesn't exists... :)

- Names of Funge-98 classes can start with "F", "C", or "FC".
  "F" means Filesystem Funge, "C" means Concurrent Funge, "FC" means... of course :)

- Filesystem98 and Concurrent98 implements instructions of Filesystem and Concurrent Funge.
  Classes whose name starts with "F", "C", "FC" are inherited from these classes,
  but these classes itself shouldn't be instantiated.
"""

from PyFunge.exception import *
from itertools import izip, count
import PyFunge.ip, PyFunge.space
import random, os, time

class FungeExec_Unefunge93(PyFunge.ip.FungeIP):
	"""Implementation of Unefunge-93 instructions.

	This class implements the following instructions:
		!, ", #, $, %, &, *, +, ,(comma), -, ., /, 0-9, :,
		<, >, ?, @, \, ], ^, _, `, g, p, |, ~.
	
	Actually Unefunge-93 doesn't exist, but it is added for convenience.
	This class cannot be instantiated.
	"""
	
	is_concurrent = False
	is_filesystem = False
	
	def __init__(self, platform):
		"""Initialize some variables.

		_command63 list contains methods which can be executed by command63().
		If you want to implement 4-or-more-dimension Funge-space,
		you should add your commands in second code block. :)
		"""
		
		PyFunge.ip.FungeIP.__init__(self, self.initspace())
		self.platform = platform
		self.delta = [1] + [0] * (self.dimension - 1)

		# function array for command63()
		self._command63 = [self.command60, self.command62]
		if self.dimension > 1:
			self._command63.extend([self.command94, self.command118])
		if self.dimension > 2:
			self._command63.extend([self.command104, self.command108])
			
		# method wrapping
		for x in xrange(48, 58):
			setattr(self, "command%d" % (x,), self.command48to57)

	def initspace(self):
		"""Create new Funge-space.
		
		Unefunge-93 class is a kind of abstract class.
		So initspace() returns just None because it shouldn't be called.. :)
		"""
		
		return None

	def getargs(self, number):
		"""Pop <number> cells from top of stack stack and return as *tuple*.
		
		If there is no cell to pop, tuple is filled with zeroes instead of items.
		"""
		
		return tuple(
			[not self.stack.is_empty() and self.stack.pop() or 0
			 for x in xrange(number)]
		)

	def getvector(self):
		"""Pop one vector from top of stack stack as tuple.

		Same as getargs(self.dimension).
		"""
		
		return self.getargs(self.dimension)
	
	def stringmode(self):
		"""Process stringmode. (when self.flag & 0x01 is true)

		For detail to stringmode, view docstring of command34() method.
		"""
		
		if self.getcell() == '"':
			self.command34()
		else:
			self.stack.push(ord(self.getcell()))

	def undefined(self):
		"""Process undefined instructions.

		In Befunge-93 (and Unefunge-93), undefined instructions don't work.
		But in Funge-98 undefined instructions are same as "r" Reflect instruction,
		therefore Unefunge98 class must override this method to reflect delta.
		"""
		
		pass
	
	def command32(self):
		"""Process " " Space instruction.

		This instruction is not normally executed, as "z" instruction in Funge-98.
		"""
		
		pass

	def command33(self):
		"""Process "!" Logical Not instruction.
		
		Stack before: <...> <n>
		Stack after:  <...> <!n>
		(bottom ... top)

		This instruction pops one cell called <n>, and pushes 1 if <n> is zero, 0 if not.
		"""
		
		if self.getargs(1)[0]:
			self.stack.push(0)
		else:
			self.stack.push(1)

	def command34(self):
		"""Process '"' Toggle Stringmode instruction.

		This instruction toggles stringmode flag.
		In stringmode, all characters won't be treated as instruction,
		but will be pushed as character code onto the stack.
		'"' instruction in stringmode will end string mode.
		"""
		
		self.flag = self.flag ^ 0x01

	def command35(self):
		"""Process "#" Trampoline instruction.

		This instruction ignores next instruction in IP's path.
		"""
		
		self.pwalk()

	def command36(self):
		"""Process "$" Pop instruction.

		Stack before: <...> <n>
		Stack after:  <...>

		This instruction pops one cell from the stack and discards it.
		Ignored if there is no cell to pop.
		"""
		
		self.getargs(1)

	def command37(self):
		"""Process "%" Remainder instruction.

		Stack before: <...> <a> <b>
		Stack after:  <...> <a%b>

		This instruction pops two cells called <b> and <a>,
		and pushes their remainder using Python's integer division. (that is, a % b)

		Raises DivisionByZeroError if b is zero or there is no cell to pop.
		"""
		
		b, a = self.getargs(2)
		if b == 0:
			raise DivisionByZeroError, "Can't divide by zero!"
		else:
			self.stack.push(a % b)

	def command38(self):
		"""Process "&" Input Integer instruction.

		Stack before: <...>
		Stack after:  <...> <inputed integer>

		This instruction inputs one integer using platform.getint(),
		and pushes the integer onto the stack.

		This instruction is platform-dependent, but it uses always buffered I/O.
		"""
		
		self.stack.push(self.platform.getint())

	def command42(self):
		"""Process "*" Multiply instruction.

		Stack before: <...> <a> <b>
		Stack after:  <...> <a*b>

		This instruction pops two cells called <b> and <a>,
		and pushes their product. (that is, a * b)

		If there is no or one cell in stack, simply pushes zero.
		"""
		
		b, a = self.getargs(2)
		self.stack.push(a * b)

	def command43(self):
		"""Process "+" Add instruction.

		Stack before: <...> <a> <b>
		Stack after:  <...> <a+b>

		This instruction pops two cell called <b> and <a>,
		and pushes their sum. (that is, a + b)

		If there is only one cell in stack, nothing happens.
		If there is no cell in stack, pushes zero.
		"""
		
		b, a = self.getargs(2)
		self.stack.push(a + b)

	def command44(self):
		"""Process "," Output Character instruction.

		Stack before: <...> <character's code>
		Stack after:  <...>

		This instruction pops one cell called <character's code>,
		and prints it as ASCII character using platform.putchar().

		If there is no cell in stack, nothing happens.
		This instruction is platform-dependent.
		"""
		
		self.platform.putchar(self.getargs(1)[0])

	def command45(self):
		"""Process "-" Subtract instruction.

		Stack before: <...> <a> <b>
		Stack after:  <...> <a-b>

		This instruction pops two cells called <b> and <a>,
		and pushes <a> - <b>.

		If there is only one cell in stack, <a> is treated as zero.
		If there is no cell in stack, pushes zero.
		"""
		
		b, a = self.getargs(2)
		self.stack.push(a - b)

	def command46(self):
		"""Process "." Output Integer instruction.

		Stack before: <...> <n>
		Stack after:  <...>

		This instruction pops one cell called <n>,
		and prints it as integer followed by space using platform.putint().

		If there is no cell in stack, nothing happens.
		This instruction is platform-dependent.
		"""
		
		self.platform.putint(self.getargs(1)[0])

	def command47(self):
		"""Process "/" Divide instruction.

		Stack before: <...> <a> <b>
		Stack after:  <...> <a//b>

		This instruction pops two cells called <b> and <a>,
		and pushes their quotient using Python's integer division. (that is, a // b)

		Raises DivisionByZeroError if b is zero or there is no cell to pop.
		"""
		
		b, a = self.getargs(2)
		if b == 0:
			raise DivisionByZeroError, "Can't divide by zero!"
		else:
			self.stack.push(a // b) # integer division

	def command48to57(self):
		"""Process "0" Push Zero to "9" Push Nine instructions.

		Stack before: <...>
		Stack after:  <...> <0, 1, 2, etc.>

		This instruction pushes the number corresponding with instruction.
		"""
		
		self.stack.push(ord(self.getcell()) - 48)

	def command58(self):
		"""Process ":" Duplicate instruction.

		Stack before: <...> <n> 
		Stack after:  <...> <n> <n>

		This instruction pushes the top cell of the stack.
		In the other words, it pops one cell called <n> and pushes twice <n>.

		If there is no cell to pop(or peek), pushes zero.
		"""
		
		try:
			self.stack.push(self.stack.peek())
		except StackUnderflowError:
			self.stack.push(0)
		except:
			raise

	def command60(self):
		"""Process "<" Go West instruction.

		This instruction set IP's delta to (-1, 0, ...).
		In the other words, it heads IP west.
		"""
		
		self.delta = [-1] + [0] * (self.dimension - 1)

	def command62(self):
		"""Process ">" Go East instruction.

		This instruction set IP's delta to (1, 0, ...).
		In the other words, it heads IP east.
		"""
		
		self.delta = [1] + [0] * (self.dimension - 1)

	def command63(self):
		"""Process "?" Go Away instruction.

		This instruction set IP's delta to "a random cardinal direction vector".
		For instance, vector will be able to be (1) or (-1) in Unefunge,
		(1, 0), (-1, 0), (0, 1), (0, -1) in Befunge, and so on.

		It uses _command63 list to decide directions able to travel.
		"""
		
		random.choice(self._command63)()

	def command64(self):
		"""Process "@" Stop instruction.

		This instruction stop and kill current IP.
		If there is only one IP -- in Non-Concurrent Funge, -- program is terminated.

		Internally it raises ExecQuitError rather than ExecStopError.
		"""
		
		raise ExecQuitError, "This program is terminated."

	def command92(self):
		"""Process "\" Swap instruction.

		Stack before: <...> <a> <b>
		Stack after:  <...> <b> <a>

		This instruction pops two cells called <b> and <a> and pushes <b> and <a>.
		When this instruction is executed, <a> and <b> are swapped with each other.

		If there is only one cell in the stack, <a> is treated as zero. (simply pushes zero)
		If there is no cell to pop, pushes two zeroes.
		"""
		
		b, a = self.getargs(2)
		self.stack.push(b)
		self.stack.push(a)

	def command95(self):
		"""Process "_" East-West If instruction.

		Stack before: <...> <n>
		Stack after:  <...>

		This instruction pops one cell called <n>,
		and heads IP west if <n> is non-zero, east if <n> is zero.

		If there is no cell to pop, <n> is treated as zero, so it heads IP east.
		"""
		
		if self.getargs(1)[0]:
			self.command60()
		else:
			self.command62()

	def command96(self):
		"""Process "`" Greater Than instruction.

		Stack before: <...> <a> <b>
		Stack after:  <...> <1 if a>b, 0 otherwise>

		This instruction pops two cells called <b> and <a>,
		and pushes 1 if <a> is greater than <b>, 0 otherwise.

		If there is only one cell in the stack, <a> is treated as zero.
		If there is no cell to pop, pushes zero. (because <a> = <b> = 0)
		"""
		
		b, a = self.getargs(2)
		if a > b:
			self.stack.push(1)
		else:
			self.stack.push(0)

	def command103(self):
		"""Process "g" Get instruction.

		Stack before: <...> <vector>
		Stack after:  <...> <cell in the vector>

		This instruction pops one vector (self.dimension-length) called <vector>,
		and pushes the code of character at given <vector> in Funge-space.

		<vector> is <x> in Unefunge, <x> <y> in Befunge, <x> <y> <z> in Trefunge.
		And <vector> is popped in the reversed order: <z>, <y>, <x> in Trefunge, etc.
		So you should push these vectors in the order.

		If there is ommitted component, the component is treated as zero.
		For instance, when there is only 3 in the stack, "g" pushes the cell at (3, 0).

		If the cell at given vector is empty or undefined, pushes 32.
		Pushed value 32 means space character.
		"""
		
		v = self.getvector()
		try:
			self.stack.push(ord(self.space.get(v)))
		except (NonDefinedCellError, EmptyCellError):
			self.stack.push(32)
		except:
			raise

	def command112(self):
		"""Process "p" Put instruction.

		Stack before: <...> <c> <vector>
		Stack after:  <...>

		This instruction is the mirror image of "g" Get instruction.
		It pops one vector called <vector>, and one cell called <c>.
		Then it stores <c> as character at given <vector> in Funge-space.

		If the cell at given vector is undefined, storing in Funge-space is ignored.
		"""
		
		v = self.getvector()
		try:
			self.space.put(v, chr(self.getargs(1)[0]))
		except (NonDefinedCellError, EmptyCellError):
			pass
		except:
			raise

	def command126(self):
		"""Process "~" Input Character instruction.

		Stack before: <...>
		Stack after:  <...> <inputed character's code>

		This instruction inputs one character using platform.getchar(),
		and pushes the character's ASCII code onto the stack.

		This instruction is platform-dependent.
		Depending on platform, it uses unbuffered or buffered I/O.
		"""

		self.stack.push(self.platform.getchar())

	def execute(self):
		"""Execute the current instruction without walking.

		This method calls stringmode(), command*(), and undefined() methods.
		If you want to walk after execute, see onestep() method.
		"""
		
		if self.flag & 0x01:
			self.stringmode()
		else:
			method = 'command%d' % (ord(self.getcell()),)
			try:
				getattr(self, method)()
			except AttributeError:
				self.undefined()
			except:
				raise

	def onestep(self):
		"""Execute the current instruction with walking.

		It is overrided in Funge-98 classes.
		"""
		
		self.execute()
		self.pwalk()

class FungeExec_Befunge93(FungeExec_Unefunge93):
	"""Implementation of Befunge-93 instructions.

	This class implements the following instructions:
		^, v, |
	"""
	
	def initspace(self):
		"""Create new Funge-space.
		
		Befunge-93 uses the fixed Funge-space, called Befunge-93-space.
		So it uses PyFunge.space.FungeStaticSpace instead of PyFunge.space.FungeSpace.
		"""
		
		return PyFunge.space.FungeStaticSpace(2, (0, 0), (79, 24))
	
	def command94(self):
		"""Process "^" Go North instruction.

		This instruction set IP's delta to (0, -1, 0, ...).
		In the other words, it heads IP north.

		This instruction requires more than 2D Funge-space. (Befunge, Trefunge, etc.)
		"""
		
		self.delta = [0, -1] + [0] * (self.dimension - 2)

	def command118(self):
		"""Process "v" Go South instruction.

		This instruction set IP's delta to (0, 1, 0, ...).
		In the other words, it heads IP south.
		
		This instruction requires more than 2D Funge-space. (Befunge, Trefunge, etc.)
		"""
		
		self.delta = [0, 1] + [0] * (self.dimension - 2)

	def command124(self):
		"""Process "|" North-South If instruction.

		Stack before: <...> <n>
		Stack after:  <...>

		This instruction pops one cell called <n>,
		and heads IP north if <n> is non-zero, south if <n> is zero.

		If there is no cell to pop, <n> is treated as zero, so it heads IP south.
		This instruction requires more than 2D Funge-space. (Befunge, Trefunge, etc.)
		"""
		
		if self.getargs(1)[0]:
			self.command94()
		else:
			self.command118()

class FungeExec_Unefunge98(FungeExec_Unefunge93):
	"""Implementation of Unefunge-98 instructions.

	This class implements the following instructions:
		', (, ), ;, A-Z, a-f, j, k, n, q, r, s, u, x, y, z, {, }
	"""

	def __init__(self, platform):
		"""Initialize some variables and wrapped methods.

		Note that methods from command65() to command91() can be modified
		by command40() and command41(). (Yes. They are fingerprint instructions!)
		"""
		
		FungeExec_Unefunge93.__init__(self, platform)

		# method wrapping
		for x in xrange(65, 91):
			setattr(self, "command%d" % (x,), self.command65to90)
		for x in xrange(97, 103):
			setattr(self, "command%d" % (x,), self.command97to102)

	def initspace(self):
		"""Create new Funge-space.

		Funge-98 uses dynamic Funge-space, so it returns the instance of FungeSpace.
		"""
		
		return PyFunge.space.FungeSpace(1)

	def undefined(self):
		"""Process undefined instructions.

		In Funge-98, undefined instructions are treated as "r" Reflect instruction.
		"""
		
		self.command114()
		
	def command39(self):
		"""Process "'" Fetch Character instruction.

		Stack before: <...>
		Stack after:  <...> <c>

		This instruction walks one step, and pushes the current cell's code onto the stack.

		If the current cell is empty, pushes 32. Pushed value 32 means space.
		This instruction requires Funge-98.
		"""
		
		self.pwalk()
		self.stack.push(ord(self.getcell()))

	def command40(self):
		"""Process "(" Load Semantics instruction.

		Stack before: <...> <fingerprint> <length>
		Stack after:  <...> <fingerprint> 1

		This instruction loads semantics of given fingerprint.
		Fingerprint is an ID of dynamically loaded extension to interpreter.

		It pops one cell called <length>, and pops <length> cells called <fingerprint>.
		<fingerprint> vector is merged by following expression:
			<value> = (...(<first> * 256 + <second>) * 256 + <third> ...) * 256 + <last>
		For instance, Funge-98 command "DCBA"4( loads 0x41424344 semantics because:
			0x41424344 = ((0x41 * 256 + 0x42) * 256 + 0x43) * 256 + 0x44.
		
		Anyway, it loads semantics of given fingerprint,
		therefore some instructions between "A" and "Z" are redefined as semantics.

		If given semantics can't be loaded, it works as "r" Reflect instruction.
		Otherwise it pushes merged fingerprint (0x41424344 in above example) and 1.
		It can be used for ")" Unload Semantics instruction.

		This instruction requires Funge-98 and corresponding fingerprint.
		Currently this instruction is unimplemented.
		"""
		
		pass #todo

	def command41(self):
		"""Process ")" Unload Semantics instruction.

		Stack before: <...> <fingerprint> <length>
		Stack after:  <...>

		This instruction unloads semantics of given fingerprint.
		It pops one cell and vector, merges <fingerprint> vector as "(" instruction.
		And it unloads given semantics.

		This instruction requires Funge-98 and corresponding fingerprint.
		Currently this instruction is unimplemented.
		"""
		
		pass #todo
	
	def command59(self):
		"""Process ";" Jump Over instruction.

		This instruction walks along the path, until next ";" appeared.
		(It can be used for comment in Funge program.)

		This instruction requires Funge-98.
		"""
		
		self.walk()
		while self.getcell() != ";":
			self.walk()

	def command65to90(self):
		"""Process "A" to "Z" Semantics instructions.

		This instructions work as "r" Reflect instruction,
		before it is redefined by "(" Load Semantics instruction.

		Initially command65() to command90() method is set to command65to90() method,
		and it will be reset to semantics method when the instruction is redefined.
		
		This instruction requires Funge-98.
		"""
		
		self.command114()

	def command97to102(self):
		"""Process "a" Push Ten to "f" Push Fifteen instructions.

		Stack before: <...>
		Stack after:  <...> <10, 11, 12, etc.>

		This instruction pushes the number corresponding with instruction.
		This instruction requires Funge-98.
		"""
		
		self.stack.push(ord(self.getcell()) - 97 + 10)
		
	def command106(self):
		"""Process "j" Jump Forward instruction.

		Stack before: <...> <n>
		Stack after:  <...>

		This instruction pops one cell called <n> and walks <n> steps along the path.
		(if <n> is negative, walks <n> steps backward. delta is not changed.)
		
		Funge command "1j" is identical to "#".
		Funge command "0j" is identical to "z". (or, nothing happens.)
		
		This instruction requires Funge-98.
		"""
		
		self.pwalk(self.stack.pop())
		
	def command107(self):
		"""Process "k" Iterate instruction.

		Stack before: <...> <n>
		Stack after:  <...>

		This instruction pops one cell called <n> and execute next instruction <n> times.
		If next instruction is " " Space or ";" Jump Over instruction, it will be ignored.

		If <n> is zero or negative, it works as "#" Trampoline instruction.
		This instruction requires Funge-98.
		"""
		
		cell = self.getcell()
		if cell == ' ' or cell == ';':
			pass
		else:
			method = 'command%d' % (ord(cell),)
			if self.has_key(method):
				for x in xrange(self.getargs(1)[0]):
					getattr(self, method)()
			else:
				self.undefined()
	
	def command110(self):
		"""Process "n" Clear Stack instruction.

		Stack before: <...>
		Stack after:

		This instruction pops all cells from the top stack of stack stack.
		This instruction requires Funge-98.
		"""
		
		self.stack.clear()
		
	def command113(self):
		"""Process "q" Quit instruction.

		This instruction immediatly terminates current program,
		whatever Concurrent Funge or Non-Concurrent Funge.

		Internally it raises ExecQuitError.
		This instruction requires Funge-98.
		"""
		
		raise ExecQuitError, "This program is terminated."
		
	def command114(self):
		"""Process "r" Reflect instruction.

		This instruction reflects delta vector, or multiplies delta vector by -1.
		After this instruction, IP will reversely walk along the path.
		
		This instruction requires Funge-98.
		"""
		
		for i in xrange(self.dimension):
			self.delta[i] = -self.delta[i]
		
	def command115(self):
		"""Process "s" Store Character instruction.

		Stack before: <...> <c>
		Stack after:  <...>

		At first this instruction pops one cell called <c>.
		And it walks one step along the path,
		then it stores <c> as character at current cell in Funge-space.

		This instruction is the mirror image of "'" Fetch Character instruction.
		This instruction requires Funge-98.
		"""
		
		current = self.getargs(1)[0]
		self.pwalk()
		self.space.put(self.position, chr(current))
		self.pwalk(-1)
		
	def command117(self):
		"""Process "u" Stack Under Stack instruction.

		Stack before: <...> <n>
		Stack after:  <...> <n cells from second stack on stack stack>

		Funge-98 supports stack stack. Of course it means stack of stack.
		The top stack of stack stack (TOSS) is same as Befunge-93's sole stack,
		and it works identically to Befunge-93's one.
		Only "u", "{", "}" instructions can manipulate stack stack.

		This instruction pops one cell called <n>,
		and transfers <n> cells from the second stack of stack stack (SOSS) to the TOSS.
		The order of transferred cells is reversed. (Initially it uses pop-push loop.)

		If <n> is negative, it transfers abs(<n>) cells from the TOSS to the SOSS.
		If <n> is zero, nothing happens.
		If <n> is greater than source stack's length, it will be filled with zero instead of cell.
		
		This instruction requires Funge-98.
		"""
		
		# NOTE: this function doesn't use FungeStack API, but uses its structure
		#       directly. so it should be modified when FungeStack is modified.
		
		if self.stack.slen() < 2:
			self.command114()
		else:
			n = self.getargs(1)[0]
			if n > 0:
				if n < self.stack.stack[-2].len():
					self.stack.stack[-1].stack.extend([0] * (n - self.stack.stack[-2].len()))
					self.stack.stack[-2].reverse()
					self.stack.stack[-1].stack.extend(self.stack.stack[-2])
					self.stack.stack[-2] = []
				else:
					buffer = self.stack.stack[-2].stack[-n:]
					buffer.reverse()
					self.stack.stack[-1].stack.extend(buffer)
					self.stack.stack[-2].stack[-n:] = []
			elif n > 0:
				if abs(n) < self.stack.stack[-1].len():
					self.stack.stack[-2].stack.extend([0] * (abs(n) - self.stack.stack[-1].len()))
					self.stack.stack[-1].reverse()
					self.stack.stack[-2].stack.extend(self.stack.stack[-1])
					self.stack.stack[-1] = []
				else:
					buffer = self.stack.stack[-1].stack[-abs(n):]
					buffer.reverse()
					self.stack.stack[-2].stack.extend(buffer)
					self.stack.stack[-1].stack[-abs(n):] = []
		
	def command120(self):
		"""Process "x" Absolute Delta instruction.

		Stack before: <...> <vector>
		Stack after:  <...>

		This instruction pops one vector called <vector>, and set IP's delta to <vector>.
		The order of <vector> is same as "g" Get or "p" Put instruction.

		If there is ommitted component, the component is treated as zero.
		This instruction requires Funge-98.
		"""
		
		self.delta = self.getvector()
		
	def command121(self):
		"""Process "y" Get Sysinfo instruction.

		Stack before: <...> <n>
		Stack after:  <...> <very many cells contain a lot of informations>

		This instruction pops one cell called <n>,
		and pushes a lot of cells contain information of system, interpreter, etc.
		See Funge-98 Specification for detail to pushed cells.

		If <n> is positive, it pushes and get <n>'th cell in the stack.
		And it pops all pushed cells. (i.e. Funge command "3y" pushes interpreter's handprint)
		If <n> is greater than the number of pushed cell, it will pushes original cell's cell.
		For instance, if "y" will pushes 2187(=3^7) cells, "7k3*k61+y" is identical to ":".
		(Because "7k3*k61+" pushes 2188 onto the stack.)
		
		This instruction requires Funge-98.
		"""
		
		mode = self.stack.pop()
		toss = self.stack.stack[-1].len()
		tm = time.localtime()

		# 20th item: enviroment variables
		self.stack.push(0)
		for env in os.environ.iteritems():
			self.stack.push_string("=".join(env))
		# 19th item: command line arguments (#todo)
		self.stack.push(0)
		self.stack.push(0)
		for cmd in [][::-1]:
			self.stack.push_string(cmd)
		# 18th item: size of each stack in stack stack
		for item in self.stack.stack[:-1]:
			self.stack.push(item.len())
		self.stack.push(toss)
		# 17th item: size of stack stack
		self.stack.push(self.stack.slen())
		# 16th item: time
		self.stack.push(tm[3] << 16 + tm[4] << 8 + tm[5])
		# 15th item: date
		self.stack.push((tm[0]-1900) << 16 + tm[1] << 8 + tm[2])
		# 14th item: greatest point (relative to the least point)
		for x in xrange(self.dimension):
			self.stack.push(self.space.rectmax[x] - self.space.rectmin[x])
		# 13th item: least point (relative to origin)
		for x in xrange(self.dimension):
			self.stack.push(self.space.rectmin[x])
		# 12th item: storage offset
		self.stack.push_list(self.offset)
		# 11th item: delta
		self.stack.push_list(self.delta)
		# 10th item: position
		self.stack.push_list(self.position)
		# 9th item: team number (concurrent funge)
		self.stack.push(self.th_team)
		# 8th item: unique id (concurrent funge)
		self.stack.push(self.th_id)
		# 7th item: number of dimensions
		self.stack.push(self.dimension)
		# 6th item: path seperator
		self.stack.push(ord(os.sep))
		# 5th item: operating paradigm (#todo; also it can be 2.)
		self.stack.push(1)
		# 4th item: implementation's version
		self.stack.push(200)
		# 3rd item: implementation's handprint
		self.stack.push(0x50794698) # "PyF\x98", likewise "tkF\x98"
		# 2nd item: number of bytes per cell
		self.stack.push(4)
		# 1st item: flags (#todo)
		flag = 0x00
		if self.is_concurrent:
			flag += 0x01
		if self.is_filesystem: # it will be support "i", "o", "=".
			flag += 0x0E
		if self.platform.is_unbufferedio:
			flag += 0x10
		self.stack.push(flag)

		if mode >= 0:
			value = self.stack.peek(mode)
			infolen = self.stack.len() - toss
			self.stack.stack[-1].stack[-infolen] = [] #todo?
	
	def command122(self):
		"""Process "z" No Operation instruction.

		This instruction is almost identical to " " Space instruction.
		(also it is explicit nop instruction.)
		
		This instruction requires Funge-98.
		"""
		
		pass
	
	def command123(self):
		"""Process "{" Begin Block instruction.

		Stack before:
		    <...> <n>
		Stack after:
		    <n cells from the TOSS> in the TOSS
		    <remained cells from the TOSS> <previous storage offset vector> in the SOSS

		This instruction pops one cell called <n>,
		pushes a new stack on the top of stack stack.
		transfers(moves) <n> cells from the SOSS (originally the TOSS) to the (new) TOSS.
		Then it pushes the storage offset onto the SOSS as "g", "p" instruction,
		and finally set IP's storage offset to a position vector of next instruction.
		The order of transferred cells is preserved.

		If <n> is negative, abs(<n>) zeroes are pushed onto the SOSS.
		If <n> is zero, no cells are transferred.
		If <n> is greater than the TOSS's length, it will be filled with zero instead of cell.
		
		This instruction requires Funge-98.
		"""
		
		# NOTE: this function doesn't use FungeStack API, but uses its structure
		#       directly. so it should be modified when FungeStack is modified.
		
		n = self.getargs(1)[0]
		self.stack.spush()
		if n >= self.stack.stack[-2].len():
			self.stack.stack[-1].stack.extend(self.stack.stack[-2].stack[-n:])
			self.stack.stack[-2].stack[-n:] = []
		elif n > 0:
			self.stack.stack[-1].stack.extend([0] * (n - self.stack.stack[-2].len()))
			self.stack.stack[-1].stack.extend(self.stack.stack[-2].stack)
			self.stack.stack[-2] = []
		elif n < 0:
			self.stack.push_list([0] * abs(n))
		for x in self.offset:
			self.stack.stack[-2].push(x)
		self.pwalk()
		self.offset = self.position
		self.pwalk(-1)
	
	def command125(self):
		"""Process "}" End Block instruction.

		Stack before:
		    <~~~> <n> in the TOSS
		    <...> <previous storage offset> in the SOSS
		Stack after:
		    <...> <n cells from the original TOSS>

		This instruction pops one cell called <n> from the TOSS,
		pops one vector from the SOSS, and set current storage offset to this vector.
		Then it transfers(moves) <n> cells from the TOSS to the SOSS,
		finally pops the TOSS from the stack stack.
		The order of transferred cells is preserved.

		If <n> is negative, it pops abs(<n>) cells from the SOSS.
		If <n> is zero, no cells are transferred.
		
		This instruction requires Funge-98.
		"""
		
		# NOTE: this function doesn't use FungeStack API, but uses its structure
		#       directly. so it should be modified when FungeStack is modified.
		
		if self.stack.slen() < 2:
			self.command114()
		else:
			n = self.getargs(1)[0]
			self.offset = list(self.getvector())
			if n >= self.stack.len():
				self.stack.stack[-2].stack.extend(self.stack.stack[-1].stack[-n:])
			elif n > 0:
				self.stack.stack[-2].stack.extend([0] * (n - self.stack.stack[-2].len()))
				self.stack.stack[-2].stack.extend(self.stack.stack[-1].stack)
			elif n < 0:
				self.stack.stack[-2].stack[-abs(n):] = []
			self.stack.spop()
	
	def onestep(self):
		"""Execute the current instruction with walking. (only for Funge-98)

		self.pwalk() in Unefunge93.onestep() method is replaced by self.walk() for wrapping.
		"""
		
		self.execute()
		self.walk()
	
class FungeExec_Befunge98(FungeExec_Unefunge98, FungeExec_Befunge93):
	"""Implementation of Befunge-98 instructions.

	This class implements the following instructions:
		[, ], w
	"""
	
	def initspace(self):
		"""Create new Funge-space."""
		
		return PyFunge.space.FungeSpace(2)

	def command91(self):
		"""Process "[" Turn Left instruction.

		This instruction turns IP's delta 90 degrees (or pi/2 radians) left.
		This instruction requires Funge-98 and more than 2D Funge-space.
		"""
		
		self.delta[0], self.delta[1] = -self.delta[1], self.delta[0]
		
	def command93(self):
		"""Process "]" Turn Right instruction.

		This instruction turns IP's delta 90 degrees (or pi/2 radians) right.
		This instruction requires Funge-98 and more than 2D Funge-space.
		"""
		
		self.delta[0], self.delta[1] = self.delta[1], -self.delta[0]
		
	def command119(self):
		"""Process "w" Compare instruction.

		Stack before: <...> <a> <b>
		Stack after:  <...>
		
		This instruction pops two cells called <b> and <a>.
		If <a> is less than <b>, it works as "[" Turn Left instruction,
		if <a> is greater than <b>, it works as "]" Turn Right instruction,
		otherwise it works as "z" No Operation instruction.

		If there is only one cell in the stack, <a> is treated as zero.
		If there is no cell to pop, nothing happens. (because <a> = <b> = 0)
		
		This instruction requires Funge-98 and more than 2D Funge-space.
		"""
		
		b, a = self.getargs(2)
		if a < b:
			self.command91()
		elif a > b:
			self.command93()
		else:
			self.command122()

class FungeExec_Trefunge98(FungeExec_Befunge98):
	"""Implementation of Trefunge-98 instructions.

	This class implements the following instructions:
		h, l, m
	"""

	def initspace(self):
		"""Create new Funge-space."""
		
		return PyFunge.space.FungeSpace(3)

	def command104(self):
		"""Process "h" Go High instruction.

		This instruction set IP's delta to (0, 0, -1, ...).
		In the other words, it heads IP high.
		
		This instruction requires Funge-98 and more than 3D Funge-space.
		"""
		
		self.delta = [0, 0, -1] + [0] * (self.dimension - 3)
		
	def command108(self):
		"""Process "l" Go Low instruction.

		This instruction set IP's delta to (0, 0, 1, ...).
		In the other words, it heads IP low.
		
		This instruction requires Funge-98 and more than 3D Funge-space.
		"""
		
		self.delta = [0, 0, 1] + [0] * (self.dimension - 3)
		
	def command109(self):
		"""Process "m" High-Low If instruction.

		Stack before: <...> <n>
		Stack after:  <...>

		This instruction pops one cell called <n>,
		and heads IP high if <n> is non-zero, low if <n> is zero.

		If there is no cell to pop, <n> is treated as zero, so it heads IP east.
		This instruction requires Funge-98 and more than 3D Funge-space.
		"""
		
		if self.getargs(1)[0]:
			self.command104()
		else:
			self.command108()

class FungeExec_Filesystem98:
	"""Implementation of Filesystem Funge instructions.

	This class implements the following instructions:
		=, i, o
	"""
	
	is_filesystem = True

	def command61(self):
		"""Process "=" Execute instruction.

		Stack before: <...> <null-terminated string>
		Stack after:  <...> <return code>

		This instruction pops null-terminated string, (i.e. 0"gnirts")
		and execute the string as command, pushes return value onto the stack.
		Executation can be done by interpreter. (But PyFunge uses os.system() call)

		If there is no zero in the stack, the string is ended with the bottommost cell.
		This instruction requires Filesystem Funge-98.
		"""
		
		command = self.stack.pop_string()
		self.stack.push(os.system(command))
	
	def command105(self):
		"""Process "i" Input File instruction.

		Stack before: <...> <position> <flag> <filename>
		Stack after:  <...> <position> <size>

		This instruction pops null-terminated string called <filename>,
		one cell called <flag>, and one vector called <position>.
		It read all contents in <filename>, and stores the contents as Funge code
		starting at <position> in Funge-space.
		
		If <flag> & 0x01 is true, it treats all EOL and FF character as normal character.
		Otherwise it treats EOL and FF character as line/plain feeding character.

		After store, it pushes two vectors called <position> and <size>.
		<position> is identical to original <position>,
		and <size> means the size of rectangle covers stored contents in Funge-space.
		It can be used in "o" Output File instruction.

		If can't open <filename>, it works as "r" Reflect instruction.
		This instruction requires Filesystem Funge-98.
		"""
		
		filename = self.stack.pop_string()
		flag = self.getargs(1)
		position = self.getvector()

		try:
			handle = file(filename, "r")
			contents = ''.join(handle.readlines())
			handle.close()

			if flag & 0x01:
				xpos = 0
				for x, char in izip(count(position[0]), contents):
					self.space.put((x,) + position[1:], x)
				else:
					size = (x - position[0] + 1,) + (1,) * (self.dimension - 1)
			else:
				size = self.space.putspace(position, contents)
			
			self.stack.push_list(position)
			self.stack.push_list(size)
		except IOError:
			self.command114()
		except:
			raise
	
	def command111(self):
		"""Process "o" Output File instruction.

		Stack before: <...> <position> <size> <flag> <filename>
		Stack after:  <...>

		This instruction pops null-terminated string called <filename>,
		one cell called <flag>, and two vector called <size> and <position>.
		Then it writes all contents from <position> to <position> + <size>
		in Funge-space into <filename>. It uses EOL and FF character to represent feeding.
		
		If <flag> & 0x01 is true, all EOL before FF, and all whitespace before EOL
		are not written into file to save the storage.

		If can't open <filename>, it works as "r" Reflect instruction.
		This instruction requires Filesystem Funge-98.
		"""
		
		filename = self.stack.pop_string()
		flag = self.getargs()
		size = self.getvector()
		position = self.getvector()

		if flag & 0x01:
			trimmed = True
		else:
			trimmed = False

		contents = self.space.getspace(position, size, trimmed)

		try:
			handle = file(filename, "w")
			handle.write(contents)
			handle.close()
		except IOError:
			self.command114()
		except:
			raise

class FungeExec_Concurrent98:
	"""Implementation of Concurrent Funge instruction.

	This class implements the following instruction:
		t
	"""
	
	is_concurrent = True
	
	def command64(self):
		"""Process "@" Stop instruction. (only for Concurrent Funge)

		It raises ExecStopError instead of ExecQuitError.
		"""
		
		raise ExecStopError, "This IP has stopped."
	
	def command116(self):
		"""Process "t" Split instruction.

		Funge-98 provides multi-threading using Concurrent Funge.
		In Concurrent Funge, all IP shares one Funge-space.
		And all instructions executed by IPs are synchonized.
		(It means that all instructions executing by IPs takes same time at the moment.)

		This instruction splits current IP into two IPs.
		Two IPs is identical to original IP, except delta.
		One IP has same IP as original IP, but another IP has reflected IP.

		The IPs doesn't destoryed before "@" or "q" instruction is executed.
		This instruction requires Concurrent Funge-98.
		"""
		
		pass #todo

#################################################################################

class FungeExec_FUnefunge98(FungeExec_Filesystem98, FungeExec_Unefunge98): """Implementation of Filesystem Unefunge-98. This class implements no new instructions."""
class FungeExec_FBefunge98(FungeExec_Filesystem98, FungeExec_Befunge98): """Implementation of Filesystem Befunge-98. This class implements no new instructions."""
class FungeExec_FTrefunge98(FungeExec_Filesystem98, FungeExec_Trefunge98): """Implementation of Filesystem Trefunge-98. This class implements no new instructions."""
class FungeExec_CUnefunge98(FungeExec_Concurrent98, FungeExec_Unefunge98): """Implementation of Concurrent Unefunge-98. This class implements no new instructions."""
class FungeExec_CBefunge98(FungeExec_Concurrent98, FungeExec_Befunge98): """Implementation of Concurrent Befunge-98. This class implements no new instructions."""
class FungeExec_CTrefunge98(FungeExec_Concurrent98, FungeExec_Trefunge98): """Implementation of Concurrent Trefunge-98. This class implements no new instructions."""
class FungeExec_FCUnefunge98(FungeExec_Filesystem98, FungeExec_Concurrent98, FungeExec_Unefunge98): """Implementation of Filesystem & Concurrent Unefunge-98. This class implements no new instructions."""
class FungeExec_FCBefunge98(FungeExec_Filesystem98, FungeExec_Concurrent98, FungeExec_Befunge98): """Implementation of Filesystem & Concurrent Befunge-98. This class implements no new instructions."""
class FungeExec_FCTrefunge98(FungeExec_Filesystem98, FungeExec_Concurrent98, FungeExec_Trefunge98): """Implementation of Filesystem & Concurrent Trefunge-98. This class implements no new instructions."""
