## PyFunge version 0.2 -- http://pyfunge.kldp.net/
## Copyright (c) 2004, Kang Seonghoon (Tokigun).
## This program is distributable under the MIT license.
##
## $Id: ip.py,v 1.3 2004/06/10 16:04:22 tokigun Exp $

"""ip module in PyFunge 0.2 -- http://pyfunge.kldp.net/
This program is distributable under the MIT license.
$Id: ip.py,v 1.3 2004/06/10 16:04:22 tokigun Exp $

This module implements Funge IP(Instruction Pointer).
There are only one class -- FungeIP.


Imported modules
================

From PyFunge packages:
    PyFunge.exception
    PyFunge.space
    PyFunge.stack


Table of hierarchy
==================

* shouldn't be instantiated.

FungeIP
 |
 +-- FungeExec_Unefunge93 from PyFunge.execute *
      |
      +-- FungeExec_Befunge93 from PyFunge.execute
      |    |
      :    :   (14 classes are ommitted)
"""

from PyFunge.exception import *
import PyFunge.space, PyFunge.stack

class FungeIP:
	"""Implementation of Funge IP(Instruction Pointer).

	PyFunge.execute.* classes are inherited from this class.
	"""

	def __init__(self, space):
		"""Initialize some variables and set walk() and pwalk() method.

		TODO: Some variables are needed -- maybe currently fingerprint database.
		"""
		
		self.dimension = space.dimension
		self.position = [0] * space.dimension
		self.delta = [0] * space.dimension
		self.offset = [0] * space.dimension
		self.flag = 0
		self.space = space
		self.stack = PyFunge.stack.FungeSStack()

		# for Concurrent Funge... (currently not used)
		self.th_team = 0
		self.th_id = 0
		
		if isinstance(self.space, PyFunge.space.FungeStaticSpace):
			self.walk = self.walk_static
			self.pwalk = self.walk_static
		else:
			self.walk = self.walk_dynamic
			self.pwalk = self.pwalk_dynamic

	def getcell(self):
		"""Get current cell as character.

		If current cell is empty, returns ' '(space).
		"""
		
		try:
			return self.space.get(self.position)
		except (NonDefinedCellError, EmptyCellError):
			return ' '
		except:
			raise

	def walk_dynamic(self, offset = 1):
		"""Walk <offset> step(s) along the path. (only for dynamic Funge-space)

		If position is outside Funge-space rectangle, position vector is wrapped.
		Wrapping function is represented by the following expression:
			pos' = min + (pos + offset * delta - min) % delta    if offset > 0
			pos' = max - (max - pos - offset * delta) % delta    if offset < 0
		
		If <offset> is negative, it walks backward along the path without reflecting delta.
		"""
		
		for i in xrange(self.dimension):
			self.position[i] += offset * self.delta[i]
			if self.position[i] > self.space.rectmax[i]:
				self.position[i] = self.space.rectmin[i] + \
					(self.position[i] - self.space.rectmin[i]) % self.delta[i]
			elif self.position[i] < self.space.rectmin[i]:
				self.position[i] = self.space.rectmax[i] - \
					(self.space.rectmax[i] - self.position[i]) % self.delta[i]

	def pwalk_dynamic(self, offset = 1):
		"""Walk <offset> step(s) along the path without wrapping.

		This method is used in instructions needs iteration or relative position.
		"""
		
		for i in xrange(self.dimension):
			self.position[i] += offset * self.delta[i]

	def walk_static(self, offset = 1):
		"""Walk <offset> step(s) along the path. (only for static Funge-space)

		Wrapping in static Funge-space is different from one in dynamic Funge-space.
		In dynamic Funge-space, IP can arrive the starting point when it walks along the path.
		But in static Funge-space it is not true. :)

		If IP uses only cardinal directions, wrapping in both Funge-space is identical.
		"""
		
		for i in xrange(self.dimension):
			self.position[i] = self.space.rectmin[i] + \
				(self.position[i] + offset * self.delta[i] - self.space.rectmin[i]) % \
				(self.space.rectmax[i] - self.space.rectmin[i] + 1)
