## PyFunge version 0.2 -- http://pyfunge.kldp.net/
## Copyright (c) 2004, Kang Seonghoon (Tokigun).
## This program is distributable under the MIT license.
##
## $Id: platform.py,v 1.4 2004/06/11 11:03:00 tokigun Exp $

"""platform module in PyFunge 0.2 -- http://pyfunge.kldp.net/
This program is distributable under the MIT license.
$Id: platform.py,v 1.4 2004/06/11 11:03:00 tokigun Exp $

This module implements platform-dependent features.
There are 8 classes, FungePlatform and FungePlatform_*.


Imported modules
================

From standard packages:
    curses - used in FungePlatform_posix
    msvcrt - used in FungePlatform_nt


Table of hierarchy
==================

* is unimplemented.

FungePlatform
 |
 +-- FungePlatform_posix
 +-- FungePlatform_nt
 |
 +-- FungePlatform_mac *
 +-- FungePlatform_os2 *
 +-- FungePlatform_ce *
 +-- FungePlatform_java *
 +-- FungePlatform_riscos *


Class naming method
===================

- "FungePlatform" is base class of the other classes.

- All other names of classes starts with prefix "FungePlatform_",
  and ends with corresponding os.type.

- In the case of other platform, (Currently, SugarCube plugin)
  probably, class name started with prefix "FungePlatform_" is prefered.
  (i.e. "FungePlatform_sugarcube")
"""

class FungePlatform:
	"""Base class of platform module.

	It implements some unbuffered input/output methods.
	"""
	
	is_unbufferedio = False
	
	def __init__(self):
		"""Initialize virtual input buffer."""
		self.getbuffer = ''

	# unbuffered input/output (not implemented)
	def getchar(self):
		"""Input a character from unbuffered stdin."""
		return -1
	def putchar(self, char):
		"""Output a character to unbuffered stdout."""
		pass

	# buffered input/output
	def _getline(self):
		"""Input a line from buffered stdin."""
		return raw_input() + '\n'
	def _getchar(self):
		"""Input a character from buffered stdin."""
		if self.getbuffer == '':
			self.getbuffer = self._getline()
		char, self.getbuffer = self.getbuffer[0], self.getbuffer[1:]
		return ord(char)
	def _putchar(self, char):
		"""Output a character to buffered stdout. (normally it is same as putchar())"""
		self.putchar(char)

	# buffered input/output with format
	def getint(self):
		"""Input a integer from buffered stdin."""
		str = ''
		flag = 1
		while not str.isdigit():
			if str == '-':
				flag = -flag
			elif str != '+':
				flag = 1
			str = chr(self._getchar())
		while str.isdigit():
			str += chr(self._getchar())
		return flag * int(str[:-1])
	def putint(self, int):
		"""Output a integer to buffered stdout."""
		for x in str(int) + ' ':
			self.putchar(ord(x))

class FungePlatform_posix(FungePlatform):
	"""Platform class for POSIX-compliant OSes using curses."""
	
	is_unbufferedio = True
	
	def __init__(self):
		"""Import module, and initialize screen."""
		FungePlatform.__init__(self)
		import curses
		self.screen = curses.initscr()
	def __del__(self):
		"""Finalize curses.

		It can use atexit module instead of __del__... :)
		"""
		import curses
		curses.endwin()
		del curses
	
	# unbuffered input/output
	def getchar(self):
		"""Input a character from unbuffered stdin."""
		return self.screen.getch()
	def putchar(self, char):
		"""Output a character to unbuffered stdout."""
		self.screen.echochar(char)

class FungePlatform_nt(FungePlatform):
	"""Platform class for Microsoft Windows using msvcrt."""
	
	is_unbufferedio = True
	
	def __init__(self):
		"""Import module."""
		FungePlatform.__init__(self)

	# unbuffered input/output
	def getchar(self):
		"""Input a character from unbuffered stdin."""
		import msvcrt
		return ord(msvcrt.getche())
	def putchar(self, char):
		"""Output a character to unbuffered stdout."""
		import msvcrt
		msvcrt.putch(chr(char))

#################################################################################
## The following classes are unimplemented... :(

class FungePlatform_mac(FungePlatform):
	"""Platform class for Macintoshi. (Unimplemnted)"""
class FungePlatform_os2(FungePlatform):
	"""Platform class for OS/2. (Unimplemnted)"""
class FungePlatform_ce(FungePlatform):
	"""Platform class for Microsoft Windows CE. (Unimplemnted)"""
class FungePlatform_java(FungePlatform):
	"""Platform class for Java. (Unimplemnted)"""
class FungePlatform_riscos(FungePlatform):
	"""Platform class for RISC OS. (Unimplemnted)"""
