## PyFunge version 0.2 -- http://pyfunge.kldp.net/
## Copyright (c) 2004, Kang Seonghoon (Tokigun).
## This program is distributable under the MIT license.
##
## $Id: space.py,v 1.4 2004/06/10 16:04:22 tokigun Exp $

"""space module in PyFunge 0.2 -- http://pyfunge.kldp.net/
This program is distributable under the MIT license.
$Id: space.py,v 1.4 2004/06/10 16:04:22 tokigun Exp $

This module implements static Befunge-93-space and dynamic Funge-98-space.
There are 2 classes, FungeSpace and FungeStaticSpace.


Imported modules
================

From PyFunge packages:
    PyFunge.exception

From standard packages:
    itertools - widely used in this module :p
    types - for checking arguments' type


Table of hierarchy
==================

FungeSpace
 |
 +-- FungeStaticSpace
"""

from PyFunge.exception import *
from types import *
from itertools import izip, count

class FungeSpace:
	"""Implementation of dynamic Funge-space, especially Lahey-space.

	It uses dictionary to implement sparse space.
	"""
	
	def __init__(self, dimension):
		"""Check the dimension, and initialize some variables.

		If dimension is wrong, raises NumberOfDimensionError.
		"""
		
		if type(dimension) is not IntType:
			raise NumberOfDimensionError("Number of dimensions should be integer.")
		if dimension < 1:
			raise NumberOfDimensionError("Lahey-space can't have less than one dimension.")
		
		self.dimension = dimension
		self.space = {}
		self.rectmax = []
		self.rectmin = []

	def check_position(self, pos):
		"""Check that the given vector pos is in this Funge-space.

		If pos is undefined vector, raises NonDefinedCellError.
		"""
		
		if type(pos) is ListType:
			pos = tuple(pos)
		elif type(pos) is not TupleType:
			raise NonDefinedCellError("Unknown position vector.")
		if len(pos) != self.dimension:
			raise NonDefinedCellError("Requested cell is non-defined cell.")
		for coord in pos:
			if type(coord) is not IntType:
				raise NonDefinedCellError("Component of position vector is not integer.")

		return pos

	def get(self, pos):
		"""Get the cell at the given vector pos in Funge-space.

		If pos is undefined vector, raises NonDefinedCellError. (in check_position())
		If the cell at pos is empty, raises EmptyCellError.
		"""
		
		pos = self.check_position(pos)
		if self.space.has_key(pos):
			return self.space[pos]
		else:
			raise EmptyCellError("Requested cell is empty.")

	def put(self, pos, char):
		"""Put the cell at the given vector pos in Funge-space, into the given character char.

		It automacally updates rectmax and rectmin vector.
		If pos is undefined vector, raises NonDefinedCellErorr. (in check_position())
		"""
		
		pos = self.check_position(pos)
		self.space[pos] = char

		if len(self.rectmax) == 0:
			self.rectmax = list(pos)
		else:
			for i in xrange(self.dimension):
				if self.rectmax[i] < pos[i]:
					self.rectmax[i] = pos[i]

		if len(self.rectmin) == 0:
			self.rectmin = list(pos)
		else:
			for i in xrange(self.dimension):
				if self.rectmin[i] > pos[i]:
					self.rectmin[i] = pos[i]

	def x_remove(self, pos): #todo: delete or refactor?
		"""Remove the cell at the given vector pos in Funge-space.

		Currently it doesn't update rectmax and rectmin vector.
		If pos is undefined vector, raises NonDefinedCellErorr. (in check_position())
		If the cell is already empty, raises EmptyCellError.
		"""
		
		pos = self.check_position(pos)
		if self.space.has_key(pos):
			del self.space[pos]
		else:
			raise EmptyCellError("Already requested cell is empty.")

	def getspace(self, pos, size, rtrim = True):
		"""Get the string represents the Funge-space from pos to pos + size using recursion.

		This method is used in "o" Output File instruction.
		If rtrim flag is true, it works as "o" instruction with <flag> = 1.
		"""
		
		pos = self.check_position(pos)
		if len(size) > 3:
			raise RuntimeError("Currently PyFunge doesn't support more than 3D funge-space.")
		elif len(size) > 2:
			str = "\f".join([
				self.getspace(pos[:2] + (z,) + pos[3:], size[:2], rtrim)
				for z in xrange(pos[2], pos[2] + size[2])
			])
			if rtrim:
				str = str.rstrip("\f")
			return str
		elif len(size) > 1:
			str = "\n".join([
				self.getspace(pos[:1] + (y,) + pos[2:], size[:1], rtrim)
				for y in xrange(pos[1], pos[1] + size[1])
			])
			if rtrim:
				str = str.rstrip("\n")
			return str
		else:
			str = ""
			for x in xrange(pos[0], pos[0] + size[0]):
				try:
					str += self.get((x,) + pos[1:])
				except (EmptyCellError, NonDefinedCellError):
					str += " "
				except:
					raise
			if rtrim:
                        	return str.rstrip(" ")
			else:
				return str

	def putspace(self, pos, str):
		"""Put the string represents the Funge-space at vector pos into Funge-space.

		Returns the size vector of rectange covers updated Funge-space cells.
		"""
		
		pos = self.check_position(pos)
		if len(pos) < 3:
			pos = pos + (0,) * (3 - len(pos))
		maxrect = list(pos)
		for z, plain in izip(count(pos[2]), str.split("\f")):
			for y, line in izip(count(pos[1]), plain.splitlines()):
				for x, char in izip(count(pos[0]), line):
					self.put(((x, y, z) + pos[3:])[:self.dimension], char)
				else:
					maxrect[0] = x + 1
			else:
				maxrect[1] = y + 1
                else:
			maxrect[2] = z + 1

		for x in xrange(self.dimension):
			maxrect[x] -= pos[x]
		return maxrect[:self.dimension]

class FungeStaticSpace(FungeSpace):
	"""Implementation of static Funge-space, such as Befunge-93-space."""
	
	def __init__(self, dimension, rectmin, rectmax):
		"""Check the dimension, and initialize some variables.

		Static Funge-space requires boundary rectmin and rectmax.
		And it can't that put into the cell outside boundary.
		"""
		
		FungeSpace.__init__(self, dimension)

		FungeSpace.check_position(self, rectmin)
		FungeSpace.check_position(self, rectmax)
		self.rectmin = rectmin
		self.rectmax = rectmax

	def check_position(self, pos):
		"""Boundary-checking version of check_position() of FungeSpace."""
		
		pos = FungeSpace.check_position(self, pos)
		for x in xrange(0, self.dimension):
			if self.rectmin[x] <= pos[x] <= self.rectmax[x]:
				pass
			else:
				raise NonDefinedCellError("Requested cell is in outside of space.")

		return pos
