## PyFunge version 0.2 -- http://pyfunge.kldp.net/
## Copyright (c) 2004, Kang Seonghoon (Tokigun).
## This program is distributable under the MIT license.
##
## $Id: stack.py,v 1.3 2004/06/10 16:04:22 tokigun Exp $

"""stack module in PyFunge 0.2 -- http://pyfunge.kldp.net/
This program is distributable under the MIT license.
$Id: stack.py,v 1.3 2004/06/10 16:04:22 tokigun Exp $

This module implements sole stack of Befunge-93 and stack stack of Funge-98.
There are 2 classes -- FungeStack and FungeSStack.

Imported modules
================

From PyFunge packages:
    PyFunge.exception

From standard packages:
    types - for checking arguments' type


Table of hierarchy
==================

FungeSStack
     :
    FungeStack
"""

from PyFunge.exception import *
from types import *

class FungeStack:
	"""Implementation of sole stack in Befunge-93."""

	def __init__(self):
		"""Initialize some variables."""
		
		self.stack = []
		self.mode_invert = False
		self.mode_queue = False

	def push(self, item):
		"""Push an item onto the stack. (with invert mode)"""
		
		if self.mode_invert:
			self.stack.insert(0, item)
		else:
			self.stack.append(item)
		
	def pop(self):
		"""Pop an item from the stack and return. (with queue mode)

		If there is no item to pop, raises StackUnderflowError.
		"""
		
		if len(self.stack) > 0:
			if self.mode_queue:
				temp = self.stack[0]
				del self.stack[0]
				return temp
			else:
				return self.stack.pop()
		else:
			raise StackUnderflowError, "There is no item to pop."

	def peek(self, offset = 0):
		"""Peek <offset+1>'th item from the stack and return.

		If <offset> is negative, it counts from the bottommost item(-1). (as Python list)
		If there is no item to peek, raises StackUnderflowError.
		"""
		
		if len(self.stack) > 0:
			return self.stack[-1-offset]
		else:
			raise StackUnderflowError, "There is no item to peek."

	def clear(self):
		"""Clear the stack."""
		
		self.stack = []

	def is_empty(self):
		"""Return True if there is no item in the stack, False otherwise."""
		
		return (len(self.stack) == 0)

	def len(self):
		"""Return a number of items in the stack."""
		
		return len(self.stack)

class FungeSStack:
	"""Implementation of stack stack in Funge-98."""

	def __init__(self):
		"""Initialize stack stack."""
		
		self.stack = [FungeStack()]

	def spush(self):
		"""Push a new stack onto the stack stack."""
		
		self.stack.append(FungeStack())

	def spop(self):
		"""Pop the top stack from the stack stack and return.

		If there is only one stack in the stack stack, raises SStackUnderflowError.
		"""
		
		if len(self.stack) > 1:
			return self.stack.pop()
		else:
			raise SStackUnderflowError, "Now stack stack has only one stack."

	def speek(self):
		"""Peek(Return) the top stack in the stack stack."""
		
		return self.stack[-1]

	def slen(self):
		"""Return the number of stacks in the stack stack."""
		
		return len(self.stack)

	def toggle_invert(self):
		"""Toggle the invertmode, and return current invertmode flag."""

		# potentional warning -- different status flag among stacks in the stack stack.
		self.stack[-1].mode_invert = not self.stack[-1].mode_invert
		return self.stack[-1].mode_invert
	
	def toggle_queue(self):
		"""Toggle the queuemode, and return current queuemode flag."""
		
		# potentional warning -- different status flag among stacks in the stack stack.
		self.stack[-1].mode_queue = not self.stack[-1].mode_queue
		return self.stack[-1].mode_queue

	def push(self, item):
		"""Push the item onto the top stack of the stack stack."""
		
		self.stack[-1].push(item)

	def pop(self):
		"""Pop the item from the top stack of the stack stack, and return it."""
		
		return self.stack[-1].pop()

	def peek(self, offset = 0):
		"""Peek <offset+1>'th item from the top stack of the stack stack and return.

		If <offset> is negative, it counts from the bottommost item(-1). (as Python list)
		"""
		
		return self.stack[-1].peek(offset)

	def clear(self):
		"""Clear the top stack of the stack stack."""
		
		self.stack[-1].stack = []
	
	def is_empty(self):
		"""Return True if there is no item in the top stack of the stack stack, False otherwise."""
		
		return (len(self.stack[-1].stack) == 0)

	def len(self):
		"""Return a number of items in the top stack of the stack stack."""
		
		return len(self.stack[-1].stack)

	def push_string(self, str):
		"""Push the null-terminated string onto the top stack of the stack stack.

		If str is not a string, raises TypeError.
		"""
		
		if type(str) is not StringType:
			raise TypeError, "Can't push non-string variable into the stack!"
		else:
			self.push(0)
			for x in xrange(len(str)-1, -1, -1):
				self.push(ord(str[x]))

	def pop_string(self):
		"""Pop the null-terminated string from the top stack of the stack stack.

		If there is no zero in the top stack, the string is ended with the bottommost cell.
		"""
		
		str = ""
		try:
			while self.peek() != 0:
				str += chr(self.pop())
			self.pop()
		except StackUnderflowError:
			pass
		except:
			raise

		return str

	def push_list(self, list):
		"""Push the listed items onto the top stack of the stack stack.

		If list is not a tuple or list, raises TypeError.
		"""
		
		if type(list) is ListType or type(list) is TupleType:
			for x in list:
				self.push(x)
		else:
			raise TypeError, "Can't push this list!"

	def pop_list(self, length): #todo?
		"""Pop the <length>-length vector from the top stack of the stack stack.

		Currently not implemented.
		PyFunge.execute.FungeExec_Unefunge93.getargs() works same.
		"""
		
		pass
